# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

import string
import datrie

WORDS = ['producers', 'pool', 'prepare', 'preview', 'prize', 'produce',
         'producer', 'progress']


def _trie():
    trie = datrie.Trie(ranges=[(chr(0), chr(127))])
    for index, word in enumerate(WORDS, 1):
        trie[word] = index

    return trie


def test_base_trie_data():
    trie = datrie.BaseTrie(string.printable)
    trie['x'] = 1
    trie['xo'] = 2
    state = datrie.BaseState(trie)
    state.walk('x')

    it = datrie.BaseIterator(state)
    it.next()
    assert it.data() == 1

    state.walk('o')

    it = datrie.BaseIterator(state)
    it.next()
    assert it.data() == 2


def test_next():
    trie = _trie()
    state = datrie.State(trie)
    it = datrie.Iterator(state)

    values = []
    while it.next():
        values.append(it.data())

    assert len(values) == 8
    assert values == [2, 3, 4, 5, 6, 7, 1, 8]


def test_next_non_root():
    trie = _trie()
    state = datrie.State(trie)
    state.walk('pr')
    it = datrie.Iterator(state)

    values = []
    while it.next():
        values.append(it.data())

    assert len(values) == 7
    assert values == [3, 4, 5, 6, 7, 1, 8]


def test_next_tail():
    trie = _trie()
    state = datrie.State(trie)
    state.walk('poo')
    it = datrie.Iterator(state)

    values = []
    while it.next():
        values.append(it.data())

    assert values == [2]


def test_keys():
    trie = _trie()
    state = datrie.State(trie)
    it = datrie.Iterator(state)

    keys = []
    while it.next():
        keys.append(it.key())

    assert keys == sorted(WORDS)


def test_keys_non_root():
    trie = _trie()
    state = datrie.State(trie)
    state.walk('pro')
    it = datrie.Iterator(state)

    keys = []
    while it.next():
        keys.append(it.key())

    assert keys == ['duce', 'ducer', 'ducers', 'gress']


def test_keys_tail():
    trie = _trie()
    state = datrie.State(trie)
    state.walk('pro')
    it = datrie.Iterator(state)

    keys = []
    while it.next():
        keys.append(it.key())

    assert keys == ['duce', 'ducer', 'ducers', 'gress']



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

import datrie


def _trie():
    trie = datrie.Trie(ranges=[(chr(0), chr(127))])
    trie['f'] = 1
    trie['fo'] = 2
    trie['fa'] = 3
    trie['faur'] = 4
    trie['fauxiiiip'] = 5
    trie['fauzox'] = 10
    trie['fauzoy'] = 20
    return trie


def test_trie_state():
    trie = _trie()
    state = datrie.State(trie)
    state.walk('f')
    assert state.data() == 1
    state.walk('o')
    assert state.data() == 2



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

import pickle
import string

import datrie

import hypothesis.strategies as st
from hypothesis import given

printable_strings = st.lists(st.text(string.printable))

@given(printable_strings)
def test_contains(words):
    trie = datrie.Trie(string.printable)
    for i, word in enumerate(set(words)):
        trie[word] = i + 1

    for i, word in enumerate(set(words)):
        assert word in trie
        assert trie[word] == trie.get(word) == i + 1

@given(printable_strings)
def test_len(words):
    trie = datrie.Trie(string.printable)
    for i, word in enumerate(set(words)):
        trie[word] = i

    assert len(trie) == len(set(words))

@given(printable_strings)
def test_pickle_unpickle(words):
    trie = datrie.Trie(string.printable)
    for i, word in enumerate(set(words)):
        trie[word] = i

    trie = pickle.loads(pickle.dumps(trie))
    for i, word in enumerate(set(words)):
        assert word in trie
        assert trie[word] == i

@given(printable_strings)
def test_pop(words):
    words = set(words)
    trie = datrie.Trie(string.printable)
    for i, word in enumerate(words):
        trie[word] = i

    for i, word in enumerate(words):
        assert trie.pop(word) == i
        assert trie.pop(word, 42) == trie.get(word, 42) == 42

@given(printable_strings)
def test_clear(words):
    words = set(words)
    trie = datrie.Trie(string.printable)
    for i, word in enumerate(words):
        trie[word] = i

    assert len(trie) == len(words)
    trie.clear()
    assert not trie
    assert len(trie) == 0

    # make sure the trie works afterwards.
    for i, word in enumerate(words):
        trie[word] = i
        assert trie[word] == i



# cython: profile=False
"""
Cython wrapper for libdatrie.
"""

from cpython.version cimport PY_MAJOR_VERSION
from cython.operator import dereference as deref
from libc.stdlib cimport malloc, free
from cpython.mem cimport PyMem_Malloc, PyMem_Realloc, PyMem_Free
from libc cimport stdio
from libc cimport string
cimport stdio_ext
cimport cdatrie

import itertools
import warnings
import sys
import tempfile

try:
    from collections.abc import MutableMapping
except ImportError:
    from collections import MutableMapping

try:
    import cPickle as pickle
except ImportError:
    import pickle

class DatrieError(Exception):
    pass

RAISE_KEY_ERROR = object()
RERAISE_KEY_ERROR = object()
DELETED_OBJECT = object()


cdef class BaseTrie:
    """
    Wrapper for libdatrie's trie.

    Keys are unicode strings, values are integers -2147483648 <= x <= 2147483647.
    """

    cdef AlphaMap alpha_map
    cdef cdatrie.Trie *_c_trie

    def __init__(self, alphabet=None, ranges=None, AlphaMap alpha_map=None, _create=True):
        """
        For efficiency trie needs to know what unicode symbols
        it should be able to store so this constructor requires
        either ``alphabet`` (a string/iterable with all allowed characters),
        ``ranges`` (a list of (begin, end) pairs, e.g. [('a', 'z')])
        or ``alpha_map`` (:class:`datrie.AlphaMap` instance).
        """
        if self._c_trie is not NULL:
            return

        if alphabet is None and ranges is None and alpha_map is None:
            raise ValueError(
                "Please provide alphabet, ranges or alpha_map argument.")

        if alpha_map is None:
            alpha_map = AlphaMap(alphabet, ranges)

        self.alpha_map = alpha_map

        if not _create:
            return

        self._c_trie = cdatrie.trie_new(alpha_map._c_alpha_map)
        if self._c_trie is NULL:
            raise MemoryError()

    def __dealloc__(self):
        if self._c_trie is not NULL:
            cdatrie.trie_free(self._c_trie)

    def update(self, other=(), **kwargs):
        if PY_MAJOR_VERSION == 2:
            if kwargs:
                raise TypeError("keyword arguments are not supported.")

        if hasattr(other, "keys"):
            for key in other:
                self[key] = other[key]
        else:
            for key, value in other:
                self[key] = value

        for key in kwargs:
            self[key] = kwargs[key]

    def clear(self):
        cdef AlphaMap alpha_map = self.alpha_map.copy()
        _c_trie = cdatrie.trie_new(alpha_map._c_alpha_map)
        if _c_trie is NULL:
            raise MemoryError()

        cdatrie.trie_free(self._c_trie)
        self._c_trie = _c_trie

    cpdef bint is_dirty(self):
        """
        Returns True if the trie is dirty with some pending changes
        and needs saving to synchronize with the file.
        """
        return cdatrie.trie_is_dirty(self._c_trie)

    def save(self, path):
        """
        Saves this trie.
        """
        with open(path, "wb", 0) as f:
            self.write(f)

    def write(self, f):
        """
        Writes a trie to a file. File-like objects without real
        file descriptors are not supported.
        """
        f.flush()

        cdef stdio.FILE* f_ptr = stdio_ext.fdopen(f.fileno(), "w")
        if f_ptr == NULL:
            raise IOError("Can't open file descriptor")

        cdef int res = cdatrie.trie_fwrite(self._c_trie, f_ptr)
        if res == -1:
            raise IOError("Can't write to file")

        stdio.fflush(f_ptr)

    def __bytes__(self):
        cdef Py_ssize_t size = cdatrie.trie_get_serialized_size(self._c_trie)
        cdef unsigned char* data = <unsigned char*> PyMem_Malloc(size)
        try:
            cdatrie.trie_serialize (self._c_trie, data)
            return <bytes> data[:size]
        finally:
            PyMem_Free(data)
        return res

    @classmethod
    def load(cls, path, alphabet=None, ranges=None, AlphaMap alpha_map=None):
        """
        Loads a trie from file.
        """
        with open(path, "rb", 0) as f:
            return cls.read(f, alphabet=alphabet, ranges=ranges, alpha_map=alpha_map)

    @classmethod
    def read(cls, f, alphabet=None, ranges=None, AlphaMap alpha_map=None):
        """
        Creates a new Trie by reading it from file.
        File-like objects without real file descriptors are not supported.

        # XXX: does it work properly in subclasses?
        """
        cdef BaseTrie trie = cls(_create=False, alphabet=alphabet, ranges=ranges, alpha_map=alpha_map)
        trie._c_trie = _load_from_file(f)
        return trie

    def __reduce__(self):
        with tempfile.NamedTemporaryFile() as f:
            self.write(f)
            f.seek(0)
            state = f.read()
            return BaseTrie, (None, None, None, False), state

    def __setstate__(self, bytes state):
        assert self._c_trie is NULL
        with tempfile.NamedTemporaryFile() as f:
            f.write(state)
            f.flush()
            f.seek(0)
            self._c_trie = _load_from_file(f)

    def __setitem__(self, unicode key, cdatrie.TrieData value):
        self._setitem(key, value)

    cdef void _setitem(self, unicode key, cdatrie.TrieData value):
        cdef cdatrie.AlphaChar* c_key = new_alpha_char_from_unicode(key)
        try:
            cdatrie.trie_store(self._c_trie, c_key, value)
        finally:
            free(c_key)

    def __getitem__(self, unicode key):
        return self._getitem(key)

    def get(self, unicode key, default=None):
        try:
            return self._getitem(key)
        except KeyError:
            return default

    cdef cdatrie.TrieData _getitem(self, unicode key) except -1:
        cdef cdatrie.TrieData data
        cdef cdatrie.AlphaChar* c_key = new_alpha_char_from_unicode(key)

        try:
            found = cdatrie.trie_retrieve(self._c_trie, c_key, &data)
        finally:
            free(c_key)

        if not found:
            raise KeyError(key)
        return data

    def __contains__(self, unicode key):
        cdef cdatrie.AlphaChar* c_key = new_alpha_char_from_unicode(key)
        try:
            return cdatrie.trie_retrieve(self._c_trie, c_key, NULL)
        finally:
            free(c_key)

    def __delitem__(self, unicode key):
        self._delitem(key)

    def pop(self, unicode key, default=None):
        try:
            value = self[key]
            self._delitem(key)
            return value
        except KeyError:
            return default

    cpdef bint _delitem(self, unicode key) except -1:
        """
        Deletes an entry for the given key from the trie. Returns
        boolean value indicating whether the key exists and is removed.
        """
        cdef cdatrie.AlphaChar* c_key = new_alpha_char_from_unicode(key)
        try:
            found = cdatrie.trie_delete(self._c_trie, c_key)
        finally:
            free(c_key)

        if not found:
            raise KeyError(key)

    @staticmethod
    cdef int len_enumerator(cdatrie.AlphaChar *key, cdatrie.TrieData key_data,
                            void *counter_ptr):
        (<int *>counter_ptr)[0] += 1
        return True

    def __len__(self):
        cdef int counter = 0
        cdatrie.trie_enumerate(self._c_trie,
                               <cdatrie.TrieEnumFunc>(self.len_enumerator),
                               &counter)
        return counter

    def __richcmp__(self, other, int op):
        if op == 2:    # ==
            if other is self:
                return True
            elif not isinstance(other, BaseTrie):
                return False

            for key in self:
                if self[key] != other[key]:
                    return False

            # XXX this can be written more efficiently via explicit iterators.
            return len(self) == len(other)
        elif op == 3:  # !=
            return not (self == other)

        raise TypeError("unorderable types: {0} and {1}".format(
            self.__class__, other.__class__))

    def setdefault(self, unicode key, cdatrie.TrieData value):
        return self._setdefault(key, value)

    cdef cdatrie.TrieData _setdefault(self, unicode key, cdatrie.TrieData value):
        cdef cdatrie.AlphaChar* c_key = new_alpha_char_from_unicode(key)
        cdef cdatrie.TrieData data

        try:
            found = cdatrie.trie_retrieve(self._c_trie, c_key, &data)
            if found:
                return data
            else:
                cdatrie.trie_store(self._c_trie, c_key, value)
                return value
        finally:
            free(c_key)

    def iter_prefixes(self, unicode key):
        '''
        Returns an iterator over the keys of this trie that are prefixes
        of ``key``.
        '''
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)
        if state == NULL:
            raise MemoryError()

        cdef int index = 1
        try:
            for char in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> char):
                    return
                if cdatrie.trie_state_is_terminal(state):
                    yield key[:index]
                index += 1
        finally:
            cdatrie.trie_state_free(state)

    def iter_prefix_items(self, unicode key):
        '''
        Returns an iterator over the items (``(key,value)`` tuples)
        of this trie that are associated with keys that are prefixes of ``key``.
        '''
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)

        if state == NULL:
            raise MemoryError()

        cdef int index = 1
        try:
            for char in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> char):
                    return
                if cdatrie.trie_state_is_terminal(state): # word is found
                    yield key[:index], cdatrie.trie_state_get_data(state)
                index += 1
        finally:
            cdatrie.trie_state_free(state)

    def iter_prefix_values(self, unicode key):
        '''
        Returns an iterator over the values of this trie that are associated
        with keys that are prefixes of ``key``.
        '''
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)

        if state == NULL:
            raise MemoryError()

        try:
            for char in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> char):
                    return
                if cdatrie.trie_state_is_terminal(state):
                    yield cdatrie.trie_state_get_data(state)
        finally:
            cdatrie.trie_state_free(state)

    def prefixes(self, unicode key):
        '''
        Returns a list with keys of this trie that are prefixes of ``key``.
        '''
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)
        if state == NULL:
            raise MemoryError()

        cdef list result = []
        cdef int index = 1
        try:
            for char in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> char):
                    break
                if cdatrie.trie_state_is_terminal(state):
                    result.append(key[:index])
                index += 1
            return result
        finally:
            cdatrie.trie_state_free(state)

    cpdef suffixes(self, unicode prefix=u''):
        """
        Returns a list of this trie's suffixes.
        If ``prefix`` is not empty, returns only the suffixes of words prefixed by ``prefix``.
        """
        cdef bint success
        cdef list res = []
        cdef BaseState state = BaseState(self)

        if prefix is not None:
            success = state.walk(prefix)
            if not success:
                return res

        cdef BaseIterator iter = BaseIterator(state)
        while iter.next():
            res.append(iter.key())

        return res

    def prefix_items(self, unicode key):
        '''
        Returns a list of the items (``(key,value)`` tuples)
        of this trie that are associated with keys that are
        prefixes of ``key``.
        '''
        return self._prefix_items(key)

    cdef list _prefix_items(self, unicode key):
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)

        if state == NULL:
            raise MemoryError()

        cdef list result = []
        cdef int index = 1
        try:
            for char in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> char):
                    break
                if cdatrie.trie_state_is_terminal(state): # word is found
                    result.append(
                        (key[:index],
                         cdatrie.trie_state_get_data(state))
                    )
                index += 1
            return result
        finally:
            cdatrie.trie_state_free(state)

    def prefix_values(self, unicode key):
        '''
        Returns a list of the values of this trie that are associated
        with keys that are prefixes of ``key``.
        '''
        return self._prefix_values(key)

    cdef list _prefix_values(self, unicode key):
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)

        if state == NULL:
            raise MemoryError()

        cdef list result = []
        try:
            for char in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> char):
                    break
                if cdatrie.trie_state_is_terminal(state): # word is found
                    result.append(cdatrie.trie_state_get_data(state))
            return result
        finally:
            cdatrie.trie_state_free(state)

    def longest_prefix(self, unicode key, default=RAISE_KEY_ERROR):
        """
        Returns the longest key in this trie that is a prefix of ``key``.

        If the trie doesn't contain any prefix of ``key``:
          - if ``default`` is given, returns it,
          - otherwise raises ``KeyError``.
        """
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)

        if state == NULL:
            raise MemoryError()

        cdef int index = 0, last_terminal_index = 0

        try:
            for ch in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> ch):
                    break

                index += 1
                if cdatrie.trie_state_is_terminal(state):
                    last_terminal_index = index

            if not last_terminal_index:
                if default is RAISE_KEY_ERROR:
                    raise KeyError(key)
                return default

            return key[:last_terminal_index]
        finally:
            cdatrie.trie_state_free(state)

    def longest_prefix_item(self, unicode key, default=RAISE_KEY_ERROR):
        """
        Returns the item (``(key,value)`` tuple) associated with the longest
        key in this trie that is a prefix of ``key``.

        If the trie doesn't contain any prefix of ``key``:
          - if ``default`` is given, returns it,
          - otherwise raises ``KeyError``.
        """
        return self._longest_prefix_item(key, default)

    cdef _longest_prefix_item(self, unicode key, default=RAISE_KEY_ERROR):
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)

        if state == NULL:
            raise MemoryError()

        cdef int index = 0, last_terminal_index = 0, data

        try:
            for ch in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> ch):
                    break

                index += 1
                if cdatrie.trie_state_is_terminal(state):
                    last_terminal_index = index
                    data = cdatrie.trie_state_get_data(state)

            if not last_terminal_index:
                if default is RAISE_KEY_ERROR:
                    raise KeyError(key)
                return default

            return key[:last_terminal_index], data

        finally:
            cdatrie.trie_state_free(state)

    def longest_prefix_value(self, unicode key, default=RAISE_KEY_ERROR):
        """
        Returns the value associated with the longest key in this trie that is
        a prefix of ``key``.

        If the trie doesn't contain any prefix of ``key``:
          - if ``default`` is given, return it
          - otherwise raise ``KeyError``
        """
        return self._longest_prefix_value(key, default)

    cdef _longest_prefix_value(self, unicode key, default=RAISE_KEY_ERROR):
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)

        if state == NULL:
            raise MemoryError()

        cdef int data = 0
        cdef char found = 0

        try:
            for ch in key:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> ch):
                    break

                if cdatrie.trie_state_is_terminal(state):
                    found = 1
                    data = cdatrie.trie_state_get_data(state)

            if not found:
                if default is RAISE_KEY_ERROR:
                    raise KeyError(key)
                return default

            return data

        finally:
            cdatrie.trie_state_free(state)

    def has_keys_with_prefix(self, unicode prefix):
        """
        Returns True if any key in the trie begins with ``prefix``.
        """
        cdef cdatrie.TrieState* state = cdatrie.trie_root(self._c_trie)
        if state == NULL:
            raise MemoryError()
        try:
            for char in prefix:
                if not cdatrie.trie_state_walk(state, <cdatrie.AlphaChar> char):
                    return False
            return True
        finally:
            cdatrie.trie_state_free(state)

    cpdef items(self, unicode prefix=None):
        """
        Returns a list of this trie's items (``(key,value)`` tuples).

        If ``prefix`` is not None, returns only the items
        associated with keys prefixed by ``prefix``.
        """
        cdef bint success
        cdef list res = []
        cdef BaseState state = BaseState(self)

        if prefix is not None:
            success = state.walk(prefix)
            if not success:
                return res

        cdef BaseIterator iter = BaseIterator(state)

        if prefix is None:
            while iter.next():
                res.append((iter.key(), iter.data()))
        else:
            while iter.next():
                res.append((prefix+iter.key(), iter.data()))

        return res

    def __iter__(self):
        cdef BaseIterator iter = BaseIterator(BaseState(self))
        while iter.next():
            yield iter.key()

    cpdef keys(self, unicode prefix=None):
        """
        Returns a list of this trie's keys.

        If ``prefix`` is not None, returns only the keys prefixed by ``prefix``.
        """
        cdef bint success
        cdef list res = []
        cdef BaseState state = BaseState(self)

        if prefix is not None:
            success = state.walk(prefix)
            if not success:
                return res

        cdef BaseIterator iter = BaseIterator(state)

        if prefix is None:
            while iter.next():
                res.append(iter.key())
        else:
            while iter.next():
                res.append(prefix+iter.key())

        return res

    cpdef values(self, unicode prefix=None):
        """
        Returns a list of this trie's values.

        If ``prefix`` is not None, returns only the values
        associated with keys prefixed by ``prefix``.
        """
        cdef bint success
        cdef list res = []
        cdef BaseState state = BaseState(self)

        if prefix is not None:
            success = state.walk(prefix)
            if not success:
                return res

        cdef BaseIterator iter = BaseIterator(state)
        while iter.next():
            res.append(iter.data())
        return res

    cdef _index_to_value(self, cdatrie.TrieData index):
        return index


cdef class Trie(BaseTrie):
    """
    Wrapper for libdatrie's trie.
    Keys are unicode strings, values are Python objects.
    """

    cdef list _values

    def __init__(self, alphabet=None, ranges=None, AlphaMap alpha_map=None, _create=True):
        """
        For efficiency trie needs to know what unicode symbols
        it should be able to store so this constructor requires
        either ``alphabet`` (a string/iterable with all allowed characters),
        ``ranges`` (a list of (begin, end) pairs, e.g. [('a', 'z')])
        or ``alpha_map`` (:class:`datrie.AlphaMap` instance).
        """
        self._values = []
        super(Trie, self).__init__(alphabet, ranges, alpha_map, _create)

    def __reduce__(self):
        with tempfile.NamedTemporaryFile() as f:
            self.write(f)
            pickle.dump(self._values, f)
            f.seek(0)
            state = f.read()
            return Trie, (None, None, None, False), state

    def __setstate__(self, bytes state):
        assert self._c_trie is NULL
        with tempfile.NamedTemporaryFile() as f:
            f.write(state)
            f.flush()
            f.seek(0)
            self._c_trie = _load_from_file(f)
            self._values = pickle.load(f)

    def __getitem__(self, unicode key):
        cdef cdatrie.TrieData index = self._getitem(key)
        return self._values[index]

    def get(self, unicode key, default=None):
        cdef cdatrie.TrieData index
        try:
            index = self._getitem(key)
            return self._values[index]
        except KeyError:
            return default

    def __setitem__(self, unicode key, object value):
        cdef cdatrie.TrieData next_index = len(self._values)
        cdef cdatrie.TrieData index = self._setdefault(key, next_index)
        if index == next_index:
            self._values.append(value)   # insert
        else:
            self._values[index] = value  # update

    def setdefault(self, unicode key, object value):
        cdef cdatrie.TrieData next_index = len(self._values)
        cdef cdatrie.TrieData index = self._setdefault(key, next_index)
        if index == next_index:
            self._values.append(value)   # insert
            return value
        else:
            return self._values[index]   # lookup

    def __delitem__(self, unicode key):
        # XXX: this could be faster (key is encoded twice here)
        cdef cdatrie.TrieData index = self._getitem(key)
        self._values[index] = DELETED_OBJECT
        self._delitem(key)

    def write(self, f):
        """
        Writes a trie to a file. File-like objects without real
        file descriptors are not supported.
        """
        super(Trie, self).write(f)
        pickle.dump(self._values, f)

    @classmethod
    def read(cls, f, alphabet=None, ranges=None, AlphaMap alpha_map=None):
        """
        Creates a new Trie by reading it from file.
        File-like objects without real file descriptors are not supported.
        """
        cdef Trie trie = super(Trie, cls).read(f, alphabet=alphabet, ranges=ranges, alpha_map=alpha_map)
        trie._values = pickle.load(f)
        return trie

    cpdef items(self, unicode prefix=None):
        """
        Returns a list of this trie's items (``(key,value)`` tuples).

        If ``prefix`` is not None, returns only the items
        associated with keys prefixed by ``prefix``.
        """

        # the following code is
        #
        #    [(k, self._values[v]) for (k,v) in BaseTrie.items(self, prefix)]
        #
        # but inlined for speed.

        cdef bint success
        cdef list res = []
        cdef BaseState state = BaseState(self)

        if prefix is not None:
            success = state.walk(prefix)
            if not success:
                return res

        cdef BaseIterator iter = BaseIterator(state)

        if prefix is None:
            while iter.next():
                res.append((iter.key(), self._values[iter.data()]))
        else:
            while iter.next():
                res.append((prefix+iter.key(), self._values[iter.data()]))

        return res

    cpdef values(self, unicode prefix=None):
        """
        Returns a list of this trie's values.

        If ``prefix`` is not None, returns only the values
        associated with keys prefixed by ``prefix``.
        """

        # the following code is
        #
        #     [self._values[v] for v in BaseTrie.values(self, prefix)]
        #
        # but inlined for speed.

        cdef list res = []
        cdef BaseState state = BaseState(self)
        cdef bint success

        if prefix is not None:
            success = state.walk(prefix)
            if not success:
                return res

        cdef BaseIterator iter = BaseIterator(state)

        while iter.next():
            res.append(self._values[iter.data()])

        return res

    def longest_prefix_item(self, unicode key, default=RAISE_KEY_ERROR):
        """
        Returns the item (``(key,value)`` tuple) associated with the longest
        key in this trie that is a prefix of ``key``.

        If the trie doesn't contain any prefix of ``key``:
          - if ``default`` is given, returns it,
          - otherwise raises ``KeyError``.
        """
        cdef res = self._longest_prefix_item(key, RERAISE_KEY_ERROR)
        if res is RERAISE_KEY_ERROR: # error
            if default is RAISE_KEY_ERROR:
                raise KeyError(key)
            return default

        return res[0], self._values[res[1]]

    def longest_prefix_value(self, unicode key, default=RAISE_KEY_ERROR):
        """
        Returns the value associated with the longest key in this trie that is
        a prefix of ``key``.

        If the trie doesn't contain any prefix of ``key``:
          - if ``default`` is given, return it
          - otherwise raise ``KeyError``
        """
        cdef res = self._longest_prefix_value(key, RERAISE_KEY_ERROR)
        if res is RERAISE_KEY_ERROR: # error
            if default is RAISE_KEY_ERROR:
                raise KeyError(key)
            return default

        return self._values[res]

    def prefix_items(self, unicode key):
        '''
        Returns a list of the items (``(key,value)`` tuples)
        of this trie that are associated with keys that are
        prefixes of ``key``.
        '''
        return [(k, self._values[v]) for (k, v) in self._prefix_items(key)]

    def iter_prefix_items(self, unicode key):
        for k, v in super(Trie, self).iter_prefix_items(key):
            yield k, self._values[v]

    def prefix_values(self, unicode key):
        '''
        Returns a list of the values of this trie that are associated
        with keys that are prefixes of ``key``.
        '''
        return [self._values[v] for v in self._prefix_values(key)]

    def iter_prefix_values(self, unicode key):
        for v in super(Trie, self).iter_prefix_values(key):
            yield self._values[v]

    cdef _index_to_value(self, cdatrie.TrieData index):
        return self._values[index]


cdef class _TrieState:
    cdef cdatrie.TrieState* _state
    cdef BaseTrie _trie

    def __cinit__(self, BaseTrie trie):
        self._state = cdatrie.trie_root(trie._c_trie)
        if self._state is NULL:
            raise MemoryError()
        self._trie = trie

    def __dealloc__(self):
        if self._state is not NULL:
            cdatrie.trie_state_free(self._state)

    cpdef walk(self, unicode to):
        cdef bint res
        for ch in to:
            if not self.walk_char(<cdatrie.AlphaChar> ch):
                return False
        return True

    cdef bint walk_char(self, cdatrie.AlphaChar char):
        """
        Walks the trie stepwise, using a given character ``char``.
        On return, the state is updated to the new state if successfully walked.
        Returns boolean value indicating the success of the walk.
        """
        return cdatrie.trie_state_walk(self._state, char)

    cpdef copy_to(self, _TrieState state):
        """ Copies trie state to another """
        cdatrie.trie_state_copy(state._state, self._state)

    cpdef rewind(self):
        """ Puts the state at root """
        cdatrie.trie_state_rewind(self._state)

    cpdef bint is_terminal(self):
        return cdatrie.trie_state_is_terminal(self._state)

    cpdef bint is_single(self):
        return cdatrie.trie_state_is_single(self._state)

    cpdef bint is_leaf(self):
        return cdatrie.trie_state_is_leaf(self._state)

    def __unicode__(self):
        return u"data:%d, term:%s, leaf:%s, single: %s" % (
            self.data(),
            self.is_terminal(),
            self.is_leaf(),
            self.is_single(),
        )

    def __repr__(self):
        return self.__unicode__()  # XXX: this is incorrect under Python 2.x


cdef class BaseState(_TrieState):
    """
    cdatrie.TrieState wrapper. It can be used for custom trie traversal.
    """
    cpdef int data(self):
        return cdatrie.trie_state_get_data(self._state)


cdef class State(_TrieState):

    def __cinit__(self, Trie trie): # this is overriden for extra type check
        self._state = cdatrie.trie_root(trie._c_trie)
        if self._state is NULL:
            raise MemoryError()
        self._trie = trie

    cpdef data(self):
        cdef cdatrie.TrieData data = cdatrie.trie_state_get_data(self._state)
        return self._trie._index_to_value(data)


cdef class _TrieIterator:
    cdef cdatrie.TrieIterator* _iter
    cdef _TrieState _root

    def __cinit__(self, _TrieState state):
        self._root = state # prevent garbage collection of state
        self._iter = cdatrie.trie_iterator_new(state._state)
        if self._iter is NULL:
            raise MemoryError()

    def __dealloc__(self):
        if self._iter is not NULL:
            cdatrie.trie_iterator_free(self._iter)

    cpdef bint next(self):
        return cdatrie.trie_iterator_next(self._iter)

    cpdef unicode key(self):
        cdef cdatrie.AlphaChar* key = cdatrie.trie_iterator_get_key(self._iter)
        try:
            return unicode_from_alpha_char(key)
        finally:
            free(key)


cdef class BaseIterator(_TrieIterator):
    """
    cdatrie.TrieIterator wrapper. It can be used for custom datrie.BaseTrie
    traversal.
    """
    cpdef cdatrie.TrieData data(self):
        return cdatrie.trie_iterator_get_data(self._iter)


cdef class Iterator(_TrieIterator):
    """
    cdatrie.TrieIterator wrapper. It can be used for custom datrie.Trie
    traversal.
    """
    def __cinit__(self, State state): # this is overriden for extra type check
        self._root = state # prevent garbage collection of state
        self._iter = cdatrie.trie_iterator_new(state._state)
        if self._iter is NULL:
            raise MemoryError()

    cpdef data(self):
        cdef cdatrie.TrieData data = cdatrie.trie_iterator_get_data(self._iter)
        return self._root._trie._index_to_value(data)


cdef (cdatrie.Trie* ) _load_from_file(f) except NULL:
    cdef int fd = f.fileno()
    cdef stdio.FILE* f_ptr = stdio_ext.fdopen(fd, "r")
    if f_ptr == NULL:
        raise IOError()
    cdef cdatrie.Trie* trie = cdatrie.trie_fread(f_ptr)
    if trie == NULL:
        raise DatrieError("Can't load trie from stream")

    cdef int f_pos = stdio.ftell(f_ptr)
    f.seek(f_pos)

    return trie

#cdef (cdatrie.Trie*) _load_from_file(path) except NULL:
#    str_path = path.encode(sys.getfilesystemencoding())
#    cdef char* c_path = str_path
#    cdef cdatrie.Trie* trie = cdatrie.trie_new_from_file(c_path)
#    if trie is NULL:
#        raise DatrieError("Can't load trie from file")
#
#    return trie


# ============================ AlphaMap & utils ================================

cdef class AlphaMap:
    """
    Alphabet map.

    For sparse data compactness, the trie alphabet set should
    be continuous, but that is usually not the case in general
    character sets. Therefore, a map between the input character
    and the low-level alphabet set for the trie is created in the
    middle. You will have to define your input character set by
    listing their continuous ranges of character codes creating a
    trie. Then, each character will be automatically assigned
    internal codes of continuous values.
    """

    cdef cdatrie.AlphaMap *_c_alpha_map

    def __cinit__(self):
        self._c_alpha_map = cdatrie.alpha_map_new()

    def __dealloc__(self):
        if self._c_alpha_map is not NULL:
            cdatrie.alpha_map_free(self._c_alpha_map)

    def __init__(self, alphabet=None, ranges=None, _create=True):
        if not _create:
            return

        if ranges is not None:
            for range in ranges:
                self.add_range(*range)

        if alphabet is not None:
            self.add_alphabet(alphabet)

    cdef AlphaMap copy(self):
        cdef AlphaMap clone = AlphaMap(_create=False)
        clone._c_alpha_map = cdatrie.alpha_map_clone(self._c_alpha_map)
        if clone._c_alpha_map is NULL:
            raise MemoryError()

        return clone

    def add_alphabet(self, alphabet):
        """
        Adds all chars from iterable to the alphabet set.
        """
        for begin, end in alphabet_to_ranges(alphabet):
            self._add_range(begin, end)

    def add_range(self, begin, end):
        """
        Add a range of character codes from ``begin`` to ``end``
        to the alphabet set.

        ``begin`` - the first character of the range;
        ``end`` - the last character of the range.
        """
        self._add_range(ord(begin), ord(end))

    cpdef _add_range(self, cdatrie.AlphaChar begin, cdatrie.AlphaChar end):
        if begin > end:
            raise DatrieError('range begin > end')
        code = cdatrie.alpha_map_add_range(self._c_alpha_map, begin, end)
        if code != 0:
            raise MemoryError()


cdef cdatrie.AlphaChar* new_alpha_char_from_unicode(unicode txt):
    """
    Converts Python unicode string to libdatrie's AlphaChar* format.
    libdatrie wants null-terminated array of 4-byte LE symbols.

    The caller should free the result of this function.
    """
    cdef int txt_len = len(txt)
    cdef int size = (txt_len + 1) * sizeof(cdatrie.AlphaChar)

    # allocate buffer
    cdef cdatrie.AlphaChar* data = <cdatrie.AlphaChar*> malloc(size)
    if data is NULL:
        raise MemoryError()

    # Copy text contents to buffer.
    # XXX: is it safe? The safe alternative is to decode txt
    # to utf32_le and then use memcpy to copy the content:
    #
    #    py_str = txt.encode('utf_32_le')
    #    cdef char* c_str = py_str
    #    string.memcpy(data, c_str, size-1)
    #
    # but the following is much (say 10x) faster and this
    # function is really in a hot spot.
    cdef int i = 0
    for char in txt:
        data[i] = <cdatrie.AlphaChar> char
        i+=1

    # Buffer must be null-terminated (last 4 bytes must be zero).
    data[txt_len] = 0
    return data


cdef unicode unicode_from_alpha_char(cdatrie.AlphaChar* key, int len=0):
    """
    Converts libdatrie's AlphaChar* to Python unicode.
    """
    cdef int length = len
    if length == 0:
        length = cdatrie.alpha_char_strlen(key)*sizeof(cdatrie.AlphaChar)
    cdef char* c_str = <char*> key
    if sys.byteorder == "little":
        return c_str[:length].decode('utf_32_le')
    else:
        return c_str[:length].decode('utf_32_be')

def to_ranges(lst):
    """
    Converts a list of numbers to a list of ranges::

    >>> numbers = [1,2,3,5,6]
    >>> list(to_ranges(numbers))
    [(1, 3), (5, 6)]
    """
    for a, b in itertools.groupby(enumerate(lst), lambda t: t[1] - t[0]):
        b = list(b)
        yield b[0][1], b[-1][1]


def alphabet_to_ranges(alphabet):
    for begin, end in to_ranges(sorted(map(ord, iter(alphabet)))):
        yield begin, end


def new(alphabet=None, ranges=None, AlphaMap alpha_map=None):
    warnings.warn('datrie.new is deprecated; please use datrie.Trie.',
                  DeprecationWarning)
    return Trie(alphabet, ranges, alpha_map)


MutableMapping.register(Trie)
MutableMapping.register(BaseTrie)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

import pickle
import random
import string
import sys
import tempfile

import datrie
import pytest


def test_trie():
    trie = datrie.Trie(string.printable)
    assert trie.is_dirty()

    assert 'foo' not in trie
    assert 'Foo' not in trie

    trie['foo'] = '5'
    assert 'foo' in trie
    assert trie['foo'] == '5'

    trie['Foo'] = 10
    assert trie['Foo'] == 10
    assert trie['foo'] == '5'
    del trie['foo']

    assert 'foo' not in trie
    assert 'Foo' in trie
    assert trie['Foo'] == 10

    with pytest.raises(KeyError):
        trie['bar']


def test_trie_invalid_alphabet():
    t = datrie.Trie('abc')
    t['a'] = 'a'
    t['b'] = 'b'
    t['c'] = 'c'

    for k in 'abc':
        assert t[k] == k

    with pytest.raises(KeyError):
        t['d']

    with pytest.raises(KeyError):
        t['e']


def test_trie_save_load():
    fd, fname = tempfile.mkstemp()
    trie = datrie.Trie(string.printable)
    trie['foobar'] = 1
    trie['foovar'] = 2
    trie['baz'] = 3
    trie['fo'] = 4
    trie['Foo'] = 'vasia'
    trie.save(fname)
    del trie

    trie2 = datrie.Trie.load(fname)
    assert trie2['foobar'] == 1
    assert trie2['baz'] == 3
    assert trie2['fo'] == 4
    assert trie2['foovar'] == 2
    assert trie2['Foo'] == 'vasia'


def test_save_load_base():
    fd, fname = tempfile.mkstemp()
    trie = datrie.BaseTrie(alphabet=string.printable)
    trie['foobar'] = 1
    trie['foovar'] = 2
    trie['baz'] = 3
    trie['fo'] = 4
    trie.save(fname)

    trie2 = datrie.BaseTrie.load(fname)
    assert trie2['foobar'] == 1
    assert trie2['baz'] == 3
    assert trie2['fo'] == 4
    assert trie2['foovar'] == 2


def test_trie_file_io():
    fd, fname = tempfile.mkstemp()

    trie = datrie.BaseTrie(string.printable)
    trie['foobar'] = 1
    trie['foo'] = 2

    extra_data = ['foo', 'bar']

    with open(fname, "wb", 0) as f:
        pickle.dump(extra_data, f)
        trie.write(f)
        pickle.dump(extra_data, f)

    with open(fname, "rb", 0) as f:
        extra_data2 = pickle.load(f)
        trie2 = datrie.BaseTrie.read(f)
        extra_data3 = pickle.load(f)

    assert extra_data2 == extra_data
    assert extra_data3 == extra_data
    assert trie2['foobar'] == 1
    assert trie2['foo'] == 2
    assert len(trie2) == len(trie)


def test_trie_unicode():
    # trie for lowercase Russian characters
    trie = datrie.Trie(ranges=[('а', 'я')])
    trie['а'] = 1
    trie['б'] = 2
    trie['аб'] = 'vasia'

    assert trie['а'] == 1
    assert trie['б'] == 2
    assert trie['аб'] == 'vasia'


def test_trie_ascii():
    trie = datrie.Trie(string.ascii_letters)
    trie['x'] = 1
    trie['y'] = 'foo'
    trie['xx'] = 2

    assert trie['x'] == 1
    assert trie['y'] == 'foo'
    assert trie['xx'] == 2


def test_trie_items():
    trie = datrie.Trie(string.ascii_lowercase)
    trie['foo'] = 10
    trie['bar'] = 'foo'
    trie['foobar'] = 30
    assert trie.values() == ['foo', 10, 30]
    assert trie.items() == [('bar', 'foo'), ('foo', 10), ('foobar', 30)]
    assert trie.keys() == ['bar', 'foo', 'foobar']


def test_trie_iter():
    trie = datrie.Trie(string.ascii_lowercase)
    assert list(trie) == []

    trie['foo'] = trie['bar'] = trie['foobar'] = 42
    assert list(trie) == ['bar', 'foo', 'foobar']


def test_trie_comparison():
    trie = datrie.Trie(string.ascii_lowercase)
    assert trie == trie
    assert trie == datrie.Trie(string.ascii_lowercase)

    other = datrie.Trie(string.ascii_lowercase)
    trie['foo'] = 42
    other['foo'] = 24
    assert trie != other

    other['foo'] = trie['foo']
    assert trie == other

    other['bar'] = 42
    assert trie != other

    with pytest.raises(TypeError):
        trie < other  # same for other comparisons


def test_trie_update():
    trie = datrie.Trie(string.ascii_lowercase)
    trie.update([("foo", 42)])
    assert trie["foo"] == 42

    trie.update({"bar": 123})
    assert trie["bar"] == 123

    if sys.version_info[0] == 2:
        with pytest.raises(TypeError):
            trie.update(bar=24)
    else:
        trie.update(bar=24)
        assert trie["bar"] == 24



def test_trie_suffixes():
    trie = datrie.Trie(string.ascii_lowercase)

    trie['pro'] = 1
    trie['prof'] = 2
    trie['product'] = 3
    trie['production'] = 4
    trie['producer'] = 5
    trie['producers'] = 6
    trie['productivity'] = 7

    assert trie.suffixes('pro') == [
        '', 'ducer', 'ducers', 'duct', 'duction', 'ductivity', 'f'
    ]


def test_trie_len():
    trie = datrie.Trie(string.ascii_lowercase)
    words = ['foo', 'f', 'faa', 'bar', 'foobar']
    for word in words:
        trie[word] = None
    assert len(trie) == len(words)

    # Calling len on an empty trie caused segfault, see #17 on GitHub.
    trie = datrie.Trie(string.ascii_lowercase)
    assert len(trie) == 0


def test_setdefault():
    trie = datrie.Trie(string.ascii_lowercase)
    assert trie.setdefault('foo', 5) == 5
    assert trie.setdefault('foo', 4) == 5
    assert trie.setdefault('foo', 5) == 5
    assert trie.setdefault('bar', 'vasia') == 'vasia'
    assert trie.setdefault('bar', 3) == 'vasia'
    assert trie.setdefault('bar', 7) == 'vasia'


class TestPrefixLookups(object):
    def _trie(self):
        trie = datrie.Trie(string.ascii_lowercase)
        trie['foo'] = 10
        trie['bar'] = 20
        trie['foobar'] = 30
        trie['foovar'] = 40
        trie['foobarzartic'] = None
        return trie

    def test_trie_keys_prefix(self):
        trie = self._trie()
        assert trie.keys('foobarz') == ['foobarzartic']
        assert trie.keys('foobarzart') == ['foobarzartic']
        assert trie.keys('foo') == ['foo', 'foobar', 'foobarzartic', 'foovar']
        assert trie.keys('foobar') == ['foobar', 'foobarzartic']
        assert trie.keys('') == [
            'bar', 'foo', 'foobar', 'foobarzartic', 'foovar'
        ]
        assert trie.keys('x') == []

    def test_trie_items_prefix(self):
        trie = self._trie()
        assert trie.items('foobarz') == [('foobarzartic', None)]
        assert trie.items('foobarzart') == [('foobarzartic', None)]
        assert trie.items('foo') == [
            ('foo', 10), ('foobar', 30), ('foobarzartic', None), ('foovar', 40)
        ]
        assert trie.items('foobar') == [('foobar', 30), ('foobarzartic', None)]
        assert trie.items('') == [
            ('bar', 20), ('foo', 10), ('foobar', 30),
            ('foobarzartic', None), ('foovar', 40)
        ]
        assert trie.items('x') == []

    def test_trie_values_prefix(self):
        trie = self._trie()
        assert trie.values('foobarz') == [None]
        assert trie.values('foobarzart') == [None]
        assert trie.values('foo') == [10, 30, None, 40]
        assert trie.values('foobar') == [30, None]
        assert trie.values('') == [20, 10, 30, None, 40]
        assert trie.values('x') == []


class TestPrefixSearch(object):

    WORDS = ['producers', 'producersz', 'pr', 'pool', 'prepare', 'preview',
             'prize', 'produce', 'producer', 'progress']

    def _trie(self):
        trie = datrie.Trie(string.ascii_lowercase)
        for index, word in enumerate(self.WORDS, 1):
            trie[word] = index
        return trie

    def test_trie_iter_prefixes(self):
        trie = self._trie()
        trie['pr'] = 'foo'

        prefixes = trie.iter_prefixes('producers')
        assert list(prefixes) == ['pr', 'produce', 'producer', 'producers']

        no_prefixes = trie.iter_prefixes('vasia')
        assert list(no_prefixes) == []

        values = trie.iter_prefix_values('producers')
        assert list(values) == ['foo', 8, 9, 1]

        no_prefixes = trie.iter_prefix_values('vasia')
        assert list(no_prefixes) == []

        items = trie.iter_prefix_items('producers')
        assert next(items) == ('pr', 'foo')
        assert next(items) == ('produce', 8)
        assert next(items) == ('producer', 9)
        assert next(items) == ('producers', 1)

        no_prefixes = trie.iter_prefix_items('vasia')
        assert list(no_prefixes) == []

    def test_trie_prefixes(self):
        trie = self._trie()

        prefixes = trie.prefixes('producers')
        assert prefixes == ['pr', 'produce', 'producer', 'producers']

        values = trie.prefix_values('producers')
        assert values == [3, 8, 9, 1]

        items = trie.prefix_items('producers')
        assert items == [('pr', 3), ('produce', 8),
                         ('producer', 9), ('producers', 1)]

        assert trie.prefixes('vasia') == []
        assert trie.prefix_values('vasia') == []
        assert trie.prefix_items('vasia') == []

    def test_has_keys_with_prefix(self):
        trie = self._trie()

        for word in self.WORDS:
            assert trie.has_keys_with_prefix(word)
            assert trie.has_keys_with_prefix(word[:-1])

        assert trie.has_keys_with_prefix('p')
        assert trie.has_keys_with_prefix('poo')
        assert trie.has_keys_with_prefix('pr')
        assert trie.has_keys_with_prefix('priz')

        assert not trie.has_keys_with_prefix('prizey')
        assert not trie.has_keys_with_prefix('ops')
        assert not trie.has_keys_with_prefix('progn')

    def test_longest_prefix(self):
        trie = self._trie()

        for word in self.WORDS:
            assert trie.longest_prefix(word) == word

        assert trie.longest_prefix('pooler') == 'pool'
        assert trie.longest_prefix('producers') == 'producers'
        assert trie.longest_prefix('progressor') == 'progress'

        assert trie.longest_prefix('paol', default=None) is None
        assert trie.longest_prefix('p', default=None) is None
        assert trie.longest_prefix('z', default=None) is None

        with pytest.raises(KeyError):
            trie.longest_prefix('z')

    def test_longest_prefix_bug(self):
        trie = self._trie()
        assert trie.longest_prefix("print") == "pr"
        assert trie.longest_prefix_value("print") == 3
        assert trie.longest_prefix_item("print") == ("pr", 3)

    def test_longest_prefix_item(self):
        trie = self._trie()

        for index, word in enumerate(self.WORDS, 1):
            assert trie.longest_prefix_item(word) == (word, index)

        assert trie.longest_prefix_item('pooler') == ('pool', 4)
        assert trie.longest_prefix_item('producers') == ('producers', 1)
        assert trie.longest_prefix_item('progressor') == ('progress', 10)

        dummy = (None, None)
        assert trie.longest_prefix_item('paol', default=dummy) == dummy
        assert trie.longest_prefix_item('p', default=dummy) == dummy
        assert trie.longest_prefix_item('z', default=dummy) == dummy

        with pytest.raises(KeyError):
            trie.longest_prefix_item('z')

    def test_longest_prefix_value(self):
        trie = self._trie()

        for index, word in enumerate(self.WORDS, 1):
            assert trie.longest_prefix_value(word) == index

        assert trie.longest_prefix_value('pooler') == 4
        assert trie.longest_prefix_value('producers') == 1
        assert trie.longest_prefix_value('progressor') == 10

        assert trie.longest_prefix_value('paol', default=None) is None
        assert trie.longest_prefix_value('p', default=None) is None
        assert trie.longest_prefix_value('z', default=None) is None

        with pytest.raises(KeyError):
            trie.longest_prefix_value('z')


def test_trie_fuzzy():
    russian = 'абвгдеёжзиклмнопрстуфхцчъыьэюя'
    alphabet = russian.upper() + string.ascii_lowercase
    words = list({
        "".join(random.choice(alphabet) for x in range(random.randint(8, 16)))
        for y in range(1000)
    })

    trie = datrie.Trie(alphabet)

    enumerated_words = list(enumerate(words))

    for index, word in enumerated_words:
        trie[word] = index

    assert len(trie) == len(words)

    random.shuffle(enumerated_words)
    for index, word in enumerated_words:
        assert word in trie, word
        assert trie[word] == index, (word, index)



# cython: profile=False
from libc cimport stdio

cdef extern from "../libdatrie/datrie/triedefs.h":
    ctypedef int AlphaChar # it should be utf32 letter
    ctypedef unsigned char TrieChar  # 1 byte
    ctypedef int TrieIndex
    ctypedef int TrieData  # int

cdef extern from "../libdatrie/datrie/alpha-map.h":

    struct AlphaMap:
        pass

    AlphaMap * alpha_map_new()
    void alpha_map_free (AlphaMap *alpha_map)
    AlphaMap * alpha_map_clone (AlphaMap *a_map)

    int alpha_map_add_range (AlphaMap *alpha_map, AlphaChar begin, AlphaChar end)
    int alpha_char_strlen (AlphaChar *str)


cdef extern from "../libdatrie/datrie/trie.h":

    ctypedef struct Trie:
        pass

    ctypedef struct TrieState:
        pass

    ctypedef struct TrieIterator:
        pass

    ctypedef int TrieData

    ctypedef bint (*TrieEnumFunc) (AlphaChar *key,
                                   TrieData key_data,
                                   void *user_data)

    int TRIE_CHAR_TERM
    int TRIE_DATA_ERROR

    # ========== GENERAL FUNCTIONS ==========

    Trie * trie_new (AlphaMap *alpha_map)

    Trie * trie_new_from_file (char *path)

    Trie * trie_fread (stdio.FILE *file)

    void trie_free (Trie *trie)

    int trie_save (Trie *trie, char *path)

    int trie_fwrite (Trie *trie, stdio.FILE *file)

    size_t  trie_get_serialized_size (Trie *trie);

    void     trie_serialize (Trie *trie, unsigned char *ptr);

    bint trie_is_dirty (Trie *trie)


    # =========== GENERAL QUERY OPERATIONS =========

    bint trie_retrieve (Trie *trie, AlphaChar *key, TrieData *o_data)

    bint trie_store (Trie *trie, AlphaChar *key, TrieData data)

    bint trie_store_if_absent (Trie *trie, AlphaChar *key, TrieData data)

    bint trie_delete (Trie *trie, AlphaChar *key)

    bint trie_enumerate (Trie *trie, TrieEnumFunc enum_func, void *user_data);

    # ======== STEPWISE QUERY OPERATIONS ========

    TrieState * trie_root (Trie *trie)


    # ========= TRIE STATE ===============

    TrieState * trie_state_clone (TrieState *s)

    void trie_state_copy (TrieState *dst, TrieState *src)

    void trie_state_free (TrieState *s)

    void trie_state_rewind (TrieState *s)

    bint trie_state_walk (TrieState *s, AlphaChar c)

    bint trie_state_is_walkable (TrieState *s, AlphaChar c)

    bint trie_state_is_terminal(TrieState * s)

    bint trie_state_is_single (TrieState *s)

    bint trie_state_is_leaf(TrieState* s)

    TrieData trie_state_get_data (TrieState *s)

    TrieData trie_state_get_data (TrieState *s)


    # ============== ITERATION ===================

    TrieIterator*   trie_iterator_new (TrieState *s)

    void            trie_iterator_free (TrieIterator *iter)

    bint            trie_iterator_next (TrieIterator *iter)

    AlphaChar *     trie_iterator_get_key (TrieIterator *iter)

    TrieData        trie_iterator_get_data (TrieIterator *iter)



#! /usr/bin/env python
"""Super-fast, efficiently stored Trie for Python."""

import glob
import os

from setuptools import setup, Extension

from Cython.Build import cythonize

LIBDATRIE_DIR = 'libdatrie'
LIBDATRIE_FILES = sorted(glob.glob(os.path.join(LIBDATRIE_DIR, "datrie", "*.c")))

DESCRIPTION = __doc__
LONG_DESCRIPTION = open('README.rst').read() + open('CHANGES.rst').read()
LICENSE = 'LGPLv2+'

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
    'Programming Language :: Cython',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: Implementation :: CPython',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Scientific/Engineering :: Information Analysis',
    'Topic :: Text Processing :: Linguistic'
]

ext_modules = cythonize(
    'src/datrie.pyx', 'src/cdatrie.pxd', 'src/stdio_ext.pxd',
    annotate=True,
    include_path=[os.path.join(os.path.dirname(os.path.abspath(__file__)), "src")],
    language_level=2
    )

for m in ext_modules:
    m.include_dirs=[LIBDATRIE_DIR]

setup(name="ant-datrie",
      version="0.8.5",
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      author='zyw',
      author_email='zyw516@gmail.com',
      license=LICENSE,
      url='https://github.com/zyw516/ant-datrie',
      classifiers=CLASSIFIERS,
      libraries=[('datrie', {
          "sources": LIBDATRIE_FILES,
          "include_dirs": [LIBDATRIE_DIR]})],
      ext_modules=ext_modules,
      python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
      setup_requires=['Cython>=0.28'],
      tests_require=["pytest", "hypothesis"])
